package com.jiayue.ssi.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import cn.hutool.core.util.NumberUtil;
import com.jiayue.ssi.annotation.OperateLog;
import com.jiayue.ssi.backenum.AuditType;
import com.jiayue.ssi.backenum.BusinessType;
import com.jiayue.ssi.constant.UserConstants;
import com.jiayue.ssi.entity.SysMenu;
import com.jiayue.ssi.service.SysMenuService;
import com.jiayue.ssi.util.RyStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import com.jiayue.ssi.annotation.InterfaceLimit;
import com.jiayue.ssi.util.ResponseVO;
import com.jiayue.ssi.util.SecurityContextUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 菜单接口
 *
 * @author xsl
 * @since 2023/03/21
 */
@RestController
@RequestMapping("/sysMenuController")
@Slf4j
public class SysMenuController {

    @Autowired
    private SysMenuService sysMenuService;

    /**
     * 获取菜单列表
     */
    @GetMapping("/list")
    @PreAuthorize("@ss.hasPermi('system:menu:list')")
    public ResponseVO list(SysMenu menu) {
        try {
            if (StringUtils.isNotEmpty(menu.getMenuName())){
                if (menu.getMenuName().length() > 50) {
                    return ResponseVO.fail("菜单名长度不能超过50个字符！");
                }
            }
            List<SysMenu> menus = sysMenuService.selectMenuList(menu, SecurityContextUtil.getSysUser().getId());
            return ResponseVO.success(menus);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取菜单列表异常");
            return ResponseVO.error(null);
        }
    }

    /**
     * 新增菜单
     */
    @PostMapping
    @OperateLog(title = "菜单管理", businessType = BusinessType.INSERT,auditType = AuditType.SYS)
    @PreAuthorize("@ss.hasPermi('system:menu:add')")
    public ResponseVO add(@RequestBody SysMenu menu) {
        if (RyStringUtils.isEmpty(menu.getMenuName())) {
            return ResponseVO.fail("菜单名称不能为空！");
        } else if (menu.getMenuName().length() > 50) {
            return ResponseVO.fail("菜单名长度不能超过50个字符！");
        }
        if (!"F".equals(menu.getMenuType())){
            if (RyStringUtils.isEmpty(menu.getPath())) {
                return ResponseVO.fail("路由地址不能为空！");
            } else if (menu.getPath().length() > 200) {
                return ResponseVO.fail("路由地址长度不能超过200个字符！");
            }
        }
        if (menu.getOrderNum() == null) {
            return ResponseVO.fail("排序不能为空！");
        } else if (!NumberUtil.isInteger(menu.getOrderNum() + "")) {
            return ResponseVO.fail("排序不是整型数值！");
        }
        if (RyStringUtils.isNotEmpty(menu.getComponent())) {
            if (menu.getComponent().length() > 200) {
                return ResponseVO.fail("组件路径长度不能超过200个字符！");
            }
        }
        if (RyStringUtils.isNotEmpty(menu.getQuery())) {
            if (menu.getQuery().length() > 200) {
                return ResponseVO.fail("路由参数长度不能超过200个字符！");
            }
        }
        if (RyStringUtils.isNotEmpty(menu.getPerms())) {
            if (menu.getPerms().length() > 100) {
                return ResponseVO.fail("权限字符长度不能超过100个字符！");
            }
        }
        if (UserConstants.NOT_UNIQUE.equals(sysMenuService.checkMenuNameUnique(menu))) {
            return ResponseVO.fail(menu.getMenuName() + "'失败，菜单名称已存在！");
        }
        if (UserConstants.YES_FRAME.equals(menu.getIsFrame()) && !RyStringUtils.ishttp(menu.getPath())) {
            return ResponseVO.fail(menu.getMenuName() + "'失败，地址必须以http(s)://开头！");
        }

        try {
            menu.setCreateBy(SecurityContextUtil.getSysUser().getUsername());
            int bo = sysMenuService.insertMenu(menu);
            if (bo == 1) {
                return ResponseVO.success("添加菜单成功");
            } else {
                log.error("添加菜单失败");
                return ResponseVO.fail("添加菜单失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("添加菜单异常");
            return ResponseVO.error(e);
        }
    }

    /**
     * 更新系统参数
     *
     * @param menu 参数
     * @return 执行结果
     */
    @PutMapping
    @OperateLog(title = "菜单管理", businessType = BusinessType.UPDATE,auditType = AuditType.SYS)
    @PreAuthorize("@ss.hasPermi('system:menu:edit')")
    public ResponseVO update(@RequestBody SysMenu menu) {
        if (menu.getMenuId() == null) {
            return ResponseVO.fail("主键为空不能修改!");
        }
        if (RyStringUtils.isEmpty(menu.getMenuName())) {
            return ResponseVO.fail("菜单名称不能为空！");
        } else if (menu.getMenuName().length() > 50) {
            return ResponseVO.fail("菜单名长度不能超过50个字符！");
        }
        if (!"F".equals(menu.getMenuType())){
            if (RyStringUtils.isEmpty(menu.getPath())) {
                return ResponseVO.fail("路由地址不能为空！");
            } else if (menu.getPath().length() > 200) {
                return ResponseVO.fail("路由地址长度不能超过200个字符！");
            }
        }
        if (menu.getOrderNum() == null) {
            return ResponseVO.fail("排序不能为空！");
        } else if (!NumberUtil.isInteger(menu.getOrderNum() + "")) {
            return ResponseVO.fail("排序不是整型数值！");
        }
        if (RyStringUtils.isNotEmpty(menu.getComponent())) {
            if (menu.getComponent().length() > 200) {
                return ResponseVO.fail("组件路径长度不能超过200个字符！");
            }
        }
        if (RyStringUtils.isNotEmpty(menu.getQuery())) {
            if (menu.getQuery().length() > 200) {
                return ResponseVO.fail("路由参数长度不能超过200个字符！");
            }
        }
        if (RyStringUtils.isNotEmpty(menu.getPerms())) {
            if (menu.getPerms().length() > 100) {
                return ResponseVO.fail("权限字符长度不能超过100个字符！");
            }
        }
        if (sysMenuService.selectMenuById(menu.getMenuId()) == null) {
            return ResponseVO.fail("非法访问不能修改！");
        }
        if (UserConstants.NOT_UNIQUE.equals(sysMenuService.checkMenuNameUnique(menu))) {
            return ResponseVO.fail("修改菜单'" + menu.getMenuName() + "'失败，菜单名称已存在！");
        } else if (UserConstants.YES_FRAME.equals(menu.getIsFrame()) && !RyStringUtils.ishttp(menu.getPath())) {
            return ResponseVO.fail("修改菜单'" + menu.getMenuName() + "'失败，地址必须以http(s)://开头");
        } else if (menu.getMenuId().equals(menu.getParentId())) {
            return ResponseVO.fail("修改菜单'" + menu.getMenuName() + "'失败，上级菜单不能选择自己");
        }

        try {
            int bo = sysMenuService.updateMenu(menu);
            if (bo == 1) {
                return ResponseVO.success("修改菜单成功");
            } else {
                log.error("修改菜单失败");
                return ResponseVO.fail("修改菜单失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("修改菜单异常");
            return ResponseVO.error(e);
        }
    }

    /**
     * 根据菜单编号获取详细信息
     */
    @GetMapping(value = "/{getDetailInfo}")
    public ResponseVO getDetailInfo(Long menuId) {
        try {
            SysMenu sysMenu = sysMenuService.selectMenuById(menuId);
            return ResponseVO.success(sysMenu);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取菜单明细异常");
            return ResponseVO.error(null);
        }
    }

    /**
     * 删除菜单信息
     */
    @DeleteMapping
    @OperateLog(title = "菜单管理", businessType = BusinessType.DELETE,auditType = AuditType.SYS)
    @PreAuthorize("@ss.hasPermi('system:menu:remove')")
    public ResponseVO delete(String menuId) {
        if (org.apache.commons.lang3.StringUtils.isEmpty(menuId)) {
            return ResponseVO.fail("删除菜单的id不能为空!");
        }
        if (sysMenuService.hasChildByMenuId(Long.parseLong(menuId))) {
            return ResponseVO.fail("存在子菜单,不允许删除");
        }
        //////////////// 角色创建后再补这个逻辑
        // if (sysMenuService.checkMenuExistRole(menuId))
        // {
        // return ResponseVO.fail("菜单已分配,不允许删除");
        // }
        try {
            int bo = sysMenuService.deleteMenuById(Long.parseLong(menuId));
            if (bo == 1) {
                return ResponseVO.success("删除成功");
            } else {
                log.error("删除失败");
                return ResponseVO.fail("删除失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("删除异常");
            return ResponseVO.error(e);
        }
    }

    /**
     * 获取菜单下拉树列表
     */
    @GetMapping("/treeselect")
    public ResponseVO treeselect(SysMenu menu) {
        List<SysMenu> menus = sysMenuService.selectMenuList(menu, SecurityContextUtil.getSysUser().getId());
        return ResponseVO.success(sysMenuService.buildMenuTreeSelect(menus));
    }

    /**
     * 加载对应角色菜单列表树
     */
    @GetMapping(value = "/roleMenuTreeselect")
    public ResponseVO roleMenuTreeselect(Long roleId) {
        List<SysMenu> menus = sysMenuService.selectMenuList(SecurityContextUtil.getSysUser().getId());
        Map<String, Object> map = new HashMap<>();
        map.put("checkedKeys", sysMenuService.selectMenuListByRoleId(roleId));
        map.put("menus", sysMenuService.buildMenuTreeSelect(menus));
        return ResponseVO.success(map);
    }
}
